@@ -36,6 +36,11 @@ struct isDouble<double>{
 #include "Rstream.hpp"
 
 /* Include the Eigen library. */
+#ifndef EIGEN_DEFAULT_DENSE_INDEX_TYPE
+#define EIGEN_DEFAULT_DENSE_INDEX_TYPE int
+#else
+#define EIGEN_DEFAULT_DENSE_INDEX_TYPE_REDEFINED 1
+#endif
 #ifdef TMB_SAFEBOUNDS
 #undef NDEBUG
 #undef eigen_assert
